import asyncio
from .client import ResourceManClient
from .manifest import build_manifest

client = ResourceManClient()

class AutoResource:
    def __init_subclass__(cls, **kwargs):
        cls._manifest = build_manifest(cls)
        cls._resource_name = cls.__name__.lower()

    def __init__(self, **kwargs):
        self.data = kwargs
        asyncio.create_task(self._register_and_submit())

    async def _register_and_submit(self):
        await client.register_manifest(self._resource_name, self.__class__._manifest)
        await client.submit_resource(self._resource_name, self.data)



from typing import Union, List, get_type_hints
from pydantic import BaseModel
from .types import *

class FieldModel(BaseModel):
    field_name: str
    field_type: Union[str, dict]
    field_description: str = ""

class ManifestModel(BaseModel):
    description: str
    fields: List[FieldModel]

def build_manifest(cls) -> ManifestModel:
    hints = get_type_hints(cls)
    fields = []
    for name, typ in hints.items():
        field = getattr(cls, name, typ)
        if isinstance(field, Enum):
            fields.append(FieldModel(field_name=name, field_type={"enum": field.choices}))
        elif typ == SimpleStr:
            fields.append(FieldModel(field_name=name, field_type="simple_str"))
        elif typ == Integer:
            fields.append(FieldModel(field_name=name, field_type="integer"))
        elif typ == Float:
            fields.append(FieldModel(field_name=name, field_type="float"))
        elif typ == Double:
            fields.append(FieldModel(field_name=name, field_type="double"))
        elif typ == Date:
            fields.append(FieldModel(field_name=name, field_type="date"))
        elif typ == Datetime:
            fields.append(FieldModel(field_name=name, field_type={"datetime": {}}))
        else:
            raise TypeError(f"Unsupported type: {typ}")
    return ManifestModel(description=f"{cls.__name__} manifest", fields=fields)



from datetime import date, datetime
from typing import Optional

class SimpleStr(str): pass
class Integer(int): pass
class Float(float): pass
class Double(float): pass
class Date(date): pass
class Datetime(datetime): pass

class Enum(str):
    def __new__(cls, *_, values: list[str], desc: Optional[str] = None):
        obj = str.__new__(cls, "enum")
        obj.choices = values
        obj.desc = desc
        return obj



from .types import (
    SimpleStr, Integer, Float, Double, Date, Datetime, Enum
)
from .core import AutoResource as Resource

__all__ = [
    "SimpleStr", "Integer", "Float", "Double", "Date", "Datetime", "Enum",
    "Resource"
]



import os
import httpx
from .manifest import ManifestModel

class ResourceManClient:
    def __init__(self, base_url: str = None):
        self._client = httpx.AsyncClient(base_url=base_url or os.getenv("RESOURCE_MAN_URL", "http://localhost:3000"))
        self._registered = set()

    async def register_manifest(self, name: str, manifest: ManifestModel):
        if name not in self._registered:
            await self._client.post(f"/resources/{name}", json=manifest.dict())
            self._registered.add(name)

    async def submit_resource(self, name: str, resource: dict):
        await self._client.post(f"/records/{name}", json=resource)

    async def close(self):
        await self._client.aclose()
